var moment = require("moment");

module.exports = function (grunt) {

	var backendModules = ['../../teknisa/man'];
	var frontendDirectories = ['utilities', 'repositories', 'services', 'controllers'];
	var subModules = [];
	var templateFiles = [
		"./",
		"./assets/js",
		"./assets/json", 
		"../../teknisa/man/reports",
		"../../teknisa/man/reports/config",
		"../backend/config/"
	];
	
	var jsDependencies = [
		'zh-grid-navigation/src/GridNavigationLibrary.min.js',
        'zh-interval-component/dist/zh-interval-component.min.js',

        'zh-grid-configuration/dist/zh-grid-configuration.min.js',
        'zh-preferences/dist/preferences.min.js',
        'zh-filter-preferences/dist/filter-preferences.min.js',
        'zh-layout-preferences/dist/layout-preferences.min.js',
        'zh-customization-preferences/dist/view-preferences.min.js',
		'zh-customization-preferences/dist/zhCustomizationsPreferences.min.js',

		'zh-crosstab/dist/zh-crosstab.min.js',
		'zh-cache-bypass/src/CacheBypass.js',
        'zh-reports/dist/zhReports.min.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.min.js',
        'zhuaclfrontend/assets/dist/main.min.js',
        'zhulog/dist/main.min.js',
        'tek-libraries/dist/main.min.js',
		'zh-charts/dist/zh-charts.min.js',
		'zh-image-uploader/dist/zh-image-uploader.min.js'
	];

	var cssDependencies = [
		'./bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css',
        './bower_components/zh-grid-navigation/src/style.css',
        './bower_components/zh-charts/dist/zh-charts.css',
		'./bower_components/zh-customizations-service/src/style/style.css',
        './bower_components/zh-select-autocomplete/src/styles/zh-select-autocomplete.css',
        './bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-image-uploader/dist/zh-image-uploader.css',
		'./assets/css/customStylesheet.css',
		'./bower_components/zh-reports/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-preferences/dist/style/style.css',
		'./bower_components/zh-preferences/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-grid-configuration/dist/style/zh-grid-configuration.css',
		'./bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.css',
		'./assets/css/odhen.css'
	];
	
	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/app.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			},
			buildLibs: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSDependenciesPaths(),
				dest: './assets/js/libs.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/bundle.min.js': ['./assets/js/libs.js', './assets/js/app.js'],
					'./assets/js/libs.min.js': './assets/js/libs.js',
					'./assets/js/app.min.js': './assets/js/app.js',
					'./assets/js/engine.min.js': './assets/js/engine.js',
					'./assets/js/environment.min.js': './assets/js/environment.js',
					'./config/Config.min.js': './config/config.js'
				}
			}
		},
		jshint: {
			all: getJSPaths().concat(["GruntFile.js"]),
			options: {
				evil: true
			}
		},
		watch: {
			pivotal: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: [ /*'jshint',*/ 'concat:js', 'concat:css']
			}
		},
		cssmin: {
			dist: {
				src: getCSSPaths(),
				dest: './assets/css/app.min.css'
			}
		},
		zhIdGenerator: {								
			options: {									
				jsonPath: getJSONPaths(),				
				menuPath: ["./assets/json/menu.json"]   
			}
		},
		buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json"
			},
			srcFolders: templateFiles
		},
		zhWords: {
		    options: {
		      serverUrl: "https://translate.teknisa.com/backend/index.php",
		      wordsPath: "./assets/json/words",
		      organization: "20",
		      project: "3694"
		    }
		}
	});

	function getJSPaths() {
		var paths = [];
		frontendDirectories.forEach(function (directory) {
			paths.push('./assets/js/' + directory + '/**/*.js');
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
			});
		});
		return paths;
	}

	function getCSSPaths() {
		var paths = [];
		cssDependencies.forEach(dependency => {
			paths.push(dependency);
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
				paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
			});
		});
		
		return paths;
	}

	function getJSDependenciesPaths() {
		var paths = [];
		jsDependencies.forEach(function(fileName) {
			paths.push('bower_components/' + fileName);
		});
		return paths;
	}

	function getJSONPaths() {
		var paths = ["./assets/json"];
		subModules.forEach(function (subModule) {
			paths.push("./assets/modules/" + subModule + "/assets/json");
		});
		return paths;
	}

	function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		parts.shift();
		parts.shift();
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");

				if(parts[0] == 'vendor') {
					srcFiles[directory][filename] = './../../backend/' + parts.join('/');
				} else {
					srcFiles[directory][filename] = './../../../' + parts.join('/');
				}
			} else {
				if(parts[0] == 'vendor') {
					srcFiles[directory].push('/../backend/' + parts.join('/'));
				} else {
					srcFiles[directory].push('/../../' + parts.join('/'));
				}
			}
		}
	}

	function backendStep () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var backendDirectories = ['config', 'datasources'];
		
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		
		backendDirectories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	}
	
	function updateIndexVersion() {
		var indexTemplatePath = './index.html';
		var indexPath = './index.html';
		var indexContent = grunt.file.read(indexTemplatePath);
		
		indexContent = indexContent.replace( /APPLICATION_VERSION/gm, moment().valueOf() + "");
		
		grunt.file.write(indexPath, indexContent);
	}
	
	function updateVersion () {
		updateIndexVersion();
	}

	grunt.registerTask('backendStep', 'build backend config files', backendStep);
	grunt.registerTask('updateVersion', 'Update version on index.html to not cache project files.', updateVersion);

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');

	grunt.registerTask('default', [ /*'jshint',*/'zhWords', 'backendStep', 'concat', 'cssmin', 'buildEnvironment',  'zhIdGenerator', 'buildAclStruct', 'uglify', 'updateVersion']);
	grunt.registerTask('buildLibs', ['concat:buildLibs']);
	grunt.registerTask('test', [ /*'jshint',*/'zhWords', 'backendStep', 'buildEnvironment:test', 'concat', 'cssmin', 'zhIdGenerator', 'buildAclStruct', 'uglify', 'updateVersion']);
	grunt.registerTask('dist', [ /*'jshint',*/'concat', 'backendStep', 'cssmin', 'uglify']);
};

